@@ -7,8 +7,8 @@
 # You can specify clean to do a make clean before building. Make clean
 # will also run configure and generate the required Makefile.
 #
-# You can specify which targets to build. If targets are specified, the
-# specified targets will be built but inst packages will not be generated.
+# This will build an smbd.noquota, smbd.profile, nmbd.profile and the
+# entire package with quota support and acl support.
 
 doclean=""
 SGI_ABI=-n32
@@ -31,6 +31,7 @@ if [ "$doclean" = "clean" ]; then
   if [ -f Makefile ]; then
     make distclean
   fi
+  rm -rf bin/*.profile bin/*.noquota
   cd ../packaging/SGI
   rm -rf bins catman html codepages swat samba.idb samba.spec
 fi
@@ -50,7 +51,7 @@ fi
 cd ../../source
 if [ "$doclean" = "clean" ]; then
   echo Create SGI specific Makefile
-  ./configure --prefix=/usr/samba --mandir=/usr/share/catman --with-acl-support --with-smbwrapper
+  ./configure --prefix=/usr/samba --sbindir='${exec_prefix}/bin' --mandir=/usr/share/catman --with-acl-support --with-quotas --with-smbwrapper
   errstat=$?
   if [ $errstat -ne 0 ]; then
     echo "Error $errstat creating Makefile\n";
@@ -63,6 +64,14 @@ fi
 #
 echo Making binaries
 
+make clean
+make "CFLAGS=-O -g3 QUOTAOBJS=smbd/noquotas.o -D WITH_PROFILE" CHECK bin/smbd
+errstat=$?
+if [ $errstat -ne 0 ]; then
+  echo "Error $errstat building profile sources\n";
+  exit $errstat;
+fi
+mv  bin/smbd bin/smbd.noquota
 make clean
 make "CFLAGS=-O -g3 -D WITH_PROFILE" CHECK bin/smbd bin/nmbd
 errstat=$?
@@ -82,13 +91,6 @@ fi
 
 cd ../packaging/SGI
 
-#
-# Don't generate packages if targets were specified
-#
-if [ "$1" != "" ]; then
-  exit 0;
-fi
-
 # generate the packages
 #
 echo Generating Inst Packages
