@@ -148,9 +148,19 @@
 #undef HAVE_SOLARIS_ACLS
 #undef HAVE_IRIX_ACLS
 #undef HAVE_AIX_ACLS
+#undef HAVE_TRU64_ACLS
 #undef HAVE_NO_ACLS
 #undef HAVE_LIBPAM
 #undef HAVE_ASPRINTF_DECL
 #undef HAVE_VASPRINTF_DECL
 #undef HAVE_SNPRINTF_DECL
 #undef HAVE_VSNPRINTF_DECL
+#undef HAVE_UNIXSOCKET
+#undef MMAP_BLACKLIST
+#undef HAVE_CUPS
+#undef WITH_LDAP_SAM
+#undef WITH_NISPLUS_SAM
+#undef WITH_SMBPASSWD_SAM
+#undef WITH_TDB_SAM
+#undef LINUX_QUOTAS_1
+#undef LINUX_QUOTAS_2
