@@ -30,7 +30,7 @@ static struct passwd *uname_string_combinations2(char *s, int offset, struct pas
  Get a users home directory.
 ****************************************************************************/
 
-char *get_home_dir(char *user)
+char *get_user_home_dir(char *user)
 {
   static struct passwd *pass;
 
@@ -84,7 +84,7 @@ BOOL map_username(char *user)
   
   f = sys_fopen(mapfile,"r");
   if (!f) {
-    DEBUG(0,("can't open username map %s\n",mapfile));
+    DEBUG(0,("can't open username map %s. Error %s\n",mapfile, strerror(errno) ));
     return False;
   }
 
@@ -169,67 +169,79 @@ static struct passwd *_Get_Pwnam(char *s)
 
 
 /****************************************************************************
- A wrapper for getpwnam() that tries with all lower and all upper case 
- if the initial name fails. Also tried with first letter capitalised
- Note that this can change user!
+ A wrapper for getpwnam().  The following variations are tried...
+    - in all lower case
+    - as transmitted IF a different case
+    - in all upper case IF that is different from the transmitted username
+    - using the lp_usernamelevel() for permutations
+ Note that this can change user! The user name is in Unix code page.
 ****************************************************************************/
 
 struct passwd *Get_Pwnam(char *user,BOOL allow_change)
 {
-  fstring user2;
-  int last_char;
-  int usernamelevel = lp_usernamelevel();
+	fstring 	user2, orig_username;
+  	int 		usernamelevel = lp_usernamelevel();
+	struct 		passwd *ret;  
 
-  struct passwd *ret;  
+	if (!user || !(*user))
+		return(NULL);
 
-  if (!user || !(*user))
-    return(NULL);
+	/* make a few copies to work with */
+	fstrcpy(orig_username, user);
+	if (!allow_change) 
+	{
+		/* allow_change was False, so make a copy and temporarily
+		   assign the char* user to the temp copy */
+		fstrcpy(user2,user);
+		user = &user2[0];
+	}
 
-  StrnCpy(user2,user,sizeof(user2)-1);
+	/* try in all lower case first as this is the most
+	   common case on UNIX systems */
+	unix_to_dos(user, True);
+	strlower(user);
+	dos_to_unix(user, True);
 
-  if (!allow_change) {
-    user = &user2[0];
-  }
+	ret = _Get_Pwnam(user);
+	if (ret)
+		return(ret);
+	
+	/* try as transmitted, but only if the original username
+	   gives us a different case */
+	if (strcmp(user, orig_username) != 0)
+	{
+		ret = _Get_Pwnam(orig_username);
+		if (ret) {
+			if (allow_change)
+				fstrcpy(user, orig_username);
 
-  ret = _Get_Pwnam(user);
-  if (ret)
-    return(ret);
+			return(ret);
+		}
+	}
 
-  strlower(user);
-  ret = _Get_Pwnam(user);
-  if (ret)
-    return(ret);
+	/* finally, try in all caps if that is a new case */
+	unix_to_dos(user, True);
+	strupper(user);
+	dos_to_unix(user, True);
 
-  strupper(user);
-  ret = _Get_Pwnam(user);
-  if (ret)
-    return(ret);
+	if (strcmp(user, orig_username) != 0)
+	{
+		ret = _Get_Pwnam(user);
+		if (ret)
+			return(ret);
+	}
 
-  /* Try with first letter capitalised. */
-  if (strlen(user) > 1)
-    strlower(user+1);  
-  ret = _Get_Pwnam(user);
-  if (ret)
-    return(ret);
+	/* Try all combinations up to usernamelevel. */
+	unix_to_dos(user, True);
+	strlower(user);
+	dos_to_unix(user, True);
 
-  /* try with last letter capitalised */
-  strlower(user);
-  last_char = strlen(user)-1;
-  user[last_char] = toupper(user[last_char]);
-  ret = _Get_Pwnam(user);
-  if (ret)
-    return(ret);
+	ret = uname_string_combinations(user, _Get_Pwnam, usernamelevel);
 
-  /* Try all combinations up to usernamelevel. */
-  strlower(user);
-  ret = uname_string_combinations(user, _Get_Pwnam, usernamelevel);
-  if (ret)
-    return(ret);
+	if (ret)
+		return(ret);
 
-  if (allow_change)
-    fstrcpy(user,user2);
-
-  return(NULL);
+	return(NULL);
 }
 
 /****************************************************************************
@@ -250,7 +262,7 @@ static BOOL user_in_netgroup_list(char *user,char *ngname)
           user, mydomain, ngname));
     DEBUG(5,("innetgr is %s\n",
           innetgr(ngname, NULL, user, mydomain)
-          ? "TRUE" : "FALSE"));
+          ? "True" : "False"));
 
     if (innetgr(ngname, NULL, user, mydomain))
       return (True);
@@ -260,36 +272,128 @@ static BOOL user_in_netgroup_list(char *user,char *ngname)
 }
 
 /****************************************************************************
- Check if a user is in a UNIX user list.
+ Check if a user is in a winbind group.
+****************************************************************************/
+  
+static BOOL user_in_winbind_group_list(char *user,char *gname, BOOL *winbind_answered)
+{
+	int num_groups;
+	int i;
+ 	gid_t *groups = NULL;
+ 	gid_t gid;
+ 	BOOL ret = False;
+ 
+ 	*winbind_answered = False;
+ 
+ 	/*
+ 	 * Get the gid's that this user belongs to.
+ 	 */
+ 
+ 	if ((num_groups = winbind_getgroups(user, 0, NULL)) == -1)
+ 		return False;
+ 
+ 	if (num_groups == 0) {
+ 		*winbind_answered = True;
+ 		return False;
+ 	}
+ 
+ 	if ((groups = (gid_t *)malloc(sizeof(gid_t) * num_groups )) == NULL) {
+ 		DEBUG(0,("user_in_winbind_group_list: malloc fail.\n"));
+ 		goto err;
+ 	}
+ 
+ 	if ((num_groups = winbind_getgroups(user, num_groups, groups)) == -1) {
+ 		DEBUG(0,("user_in_winbind_group_list: second winbind_getgroups call \
+failed with error %s\n", strerror(errno) ));
+ 		goto err;
+	}
+ 
+ 	/*
+ 	 * Now we have the gid list for this user - convert the gname
+ 	 * to a gid_t via either winbind or the local UNIX lookup and do the comparison.
+ 	 */
+ 
+	if ((gid = nametogid(gname)) == (gid_t)-1) {
+ 		DEBUG(0,("user_in_winbind_group_list: winbind_lookup_name for group %s failed.\n",
+ 			gname ));
+ 		goto err;
+ 	}
+ 
+ 	for (i = 0; i < num_groups; i++) {
+ 		if (gid == groups[i]) {
+ 			ret = True;
+ 			break;
+ 		}
+ 	}
+ 
+ 	*winbind_answered = True;
+ 	safe_free(groups);
+ 	return ret;
+ 
+   err:
+ 
+ 	*winbind_answered = False;
+ 	safe_free(groups);
+ 	return False;
+}	      
+ 
+/****************************************************************************
+ Check if a user is in a UNIX group.
 ****************************************************************************/
 
-static BOOL user_in_group_list(char *user,char *gname)
+static BOOL user_in_unix_group_list(char *user,char *gname)
 {
-#ifdef HAVE_GETGRNAM 
-  struct group *gptr;
-  char **member;  
-  struct passwd *pass = Get_Pwnam(user,False);
+	struct group *gptr;
+	char **member;  
+	struct passwd *pass = Get_Pwnam(user,False);
 
-  if (pass) { 
-    gptr = getgrgid(pass->pw_gid);
-    if (gptr && strequal(gptr->gr_name,gname))
-      return(True); 
-  } 
+	DEBUG(10,("user_in_unix_group_list: checking user %s in group %s\n", user, gname));
 
-  gptr = (struct group *)getgrnam(gname);
+ 	/*
+ 	 * We need to check the users primary group as this
+ 	 * group is implicit and often not listed in the group database.
+ 	 */
+ 
+ 	if (pass) {
+ 		gptr = getgrgid(pass->pw_gid);
+ 		if (gptr && strequal(gptr->gr_name,gname)) {
+ 			DEBUG(10,("user_in_unix_group_list: group %s is primary group.\n", gname ));
+ 			return True;
+ 		}
+ 	}
+ 
+ 	if ((gptr = (struct group *)getgrnam(gname)) == NULL) {
+ 		DEBUG(10,("user_in_unix_group_list: no such group %s\n", gname ));
+ 		return False;
+ 	}
+ 
+ 	member = gptr->gr_mem;
+  	while (member && *member) {
+ 		DEBUG(10,("user_in_unix_group_list: checking user %s against member %s\n", user, *member ));
+  		if (strequal(*member,user)) {
+  			return(True);
+  		}
+		member++;
+	}
 
-  if (gptr) {
-    member = gptr->gr_mem;
-    while (member && *member) {
-      if (strequal(*member,user))
-        return(True);
-      member++;
-    }
-  }
-#endif /* HAVE_GETGRNAM */
-  return False;
+	return False;
 }	      
 
+/****************************************************************************
+ Check if a user is in a group list. Ask winbind first, then use UNIX.
+****************************************************************************/
+
+BOOL user_in_group_list(char *user,char *gname)
+{
+	BOOL winbind_answered = False;
+	BOOL ret = user_in_winbind_group_list(user, gname, &winbind_answered);
+
+	if (winbind_answered)
+		return ret;
+
+	return user_in_unix_group_list(user, gname);	
+}
+
 /****************************************************************************
  Check if a user is in a user list - can check combinations of UNIX
  and netgroup lists.
@@ -300,6 +404,8 @@ BOOL user_in_list(char *user,char *list)
   pstring tok;
   char *p=list;
 
+  DEBUG(10,("user_in_list: checking user %s in list %s\n", user, list));
+
   while (next_token(&p,tok,LIST_SEP, sizeof(tok))) {
     /*
      * Check raw username.
@@ -344,7 +450,7 @@ BOOL user_in_list(char *user,char *list)
 
     } else if (*tok == '&') {
 
-      if(tok[1] == '&') {
+      if(tok[1] == '+') {
         /*
          * Search netgroup list followed by UNIX list.
          */
@@ -419,3 +525,32 @@ static struct passwd * uname_string_combinations(char *s,struct passwd * (*fn)(c
   }
   return(NULL);
 }
+
+
+/****************************************************************************
+these wrappers allow appliance mode to work. In appliance mode the username
+takes the form DOMAIN/user
+****************************************************************************/
+struct passwd *smb_getpwnam(char *user, BOOL allow_change)
+{
+	struct passwd *pw;
+	char *p;
+	char *sep;
+	extern pstring global_myname;
+
+	pw = Get_Pwnam(user, allow_change);
+	if (pw) return pw;
+
+	/* if it is a domain qualified name and it isn't in our password
+	   database but the domain portion matches our local machine name then
+	   lookup just the username portion locally */
+	sep = lp_winbind_separator();
+	if (!sep || !*sep) sep = "\\";
+	p = strchr(user,*sep);
+	if (p && 
+	    strncasecmp(global_myname, user, strlen(global_myname))==0) {
+		return Get_Pwnam(p+1, allow_change);
+	}
+
+	return NULL;
+}
