@@ -732,7 +732,7 @@ static void usage(char *pname)
           strupper(global_myname);
           break;
         case 'l':
-          slprintf(debugf,sizeof(debugf)-1, "%s.nmb",optarg);
+          slprintf(debugf, sizeof(debugf)-1, "%s/log.nmbd", optarg);
           break;
         case 'a':
           append_log = True;
@@ -776,6 +776,13 @@ static void usage(char *pname)
   if ( !reload_nmbd_services(False) )
     return(-1);
 
+#ifdef WITH_PROFILE
+  if (!profile_setup(False)) {
+	DEBUG(0,("ERROR: failed to setup profiling shared memory\n"));
+	return -1;
+  }
+#endif /* WITH_PROFILE */
+
   codepage_initialise(lp_client_code_page());
 
   if(!init_structs())
