@@ -54,13 +54,14 @@ void process_logon_packet(struct packet_struct *p,char *buf,int len,
   pstring ascuser;
   char *unicomp; /* Unicode computer name. */
 
+  START_PROFILE(domain_logon);
   memset(outbuf, 0, sizeof(outbuf));
 
   if (!lp_domain_logons())
   {
     DEBUG(3,("process_logon_packet: Logon packet received from IP %s and domain \
 logons are not enabled.\n", inet_ntoa(p->ip) ));
-    return;
+    goto done;
   }
 
   pstrcpy(my_name, global_myname);
@@ -113,6 +114,12 @@ logons are not enabled.\n", inet_ntoa(p->ip) ));
       char *q = buf + 2;
       char *machine = q;
 
+      if (!lp_domain_master())
+      {  
+         /* We're not Primary Domain Controller -- ignore this */
+         goto done;
+      }
+
       getdc = skip_string(machine,1);
       q = skip_string(getdc,1);
       q = ALIGN2(q, buf);
@@ -186,7 +193,7 @@ reporting %s domain %s 0x%x ntversion=%x lm_nt token=%x lm_20 token=%x\n",
                   dgram->source_name.name,
                   dgram->source_name.name_type,
                   p->ip, *iface_ip(p->ip), p->port);  
-      return;
+      goto done;
     }
 
     case SAMLOGON:
@@ -281,7 +288,9 @@ reporting %s domain %s 0x%x ntversion=%x lm_nt token=%x lm_20 token=%x\n",
     default:
     {
       DEBUG(3,("process_logon_packet: Unknown domain request %d\n",code));
-      return;
+      goto done;
     }
   }
+done:
+  END_PROFILE(domain_logon);
 }
