@@ -530,7 +530,7 @@ BOOL lsa_io_q_enum_trust_dom(char *desc, LSA_Q_ENUM_TRUST_DOM *q_e,
  Inits an LSA_R_ENUM_TRUST_DOM structure.
 ********************************************************************/
 
-void init_r_enum_trust_dom(LSA_R_ENUM_TRUST_DOM *r_e, uint32 enum_context, 
+void init_r_enum_trust_dom(TALLOC_CTX *ctx, LSA_R_ENUM_TRUST_DOM *r_e, uint32 enum_context, 
 			   char *domain_name, DOM_SID *domain_sid,
                            uint32 status)
 {
@@ -545,14 +545,14 @@ void init_r_enum_trust_dom(LSA_R_ENUM_TRUST_DOM *r_e, uint32 enum_context,
                 r_e->ptr_enum_domains = 1;
                 r_e->num_domains2 = 1;
 		
-		if (!(r_e->hdr_domain_name = (UNIHDR2 *)
-		      malloc(sizeof(UNIHDR2)))) return;
+		if (!(r_e->hdr_domain_name = (UNIHDR2 *)talloc(ctx,sizeof(UNIHDR2))))
+			return;
 
-		if (!(r_e->uni_domain_name = (UNISTR2 *)
-		      malloc(sizeof(UNISTR2)))) return;
+		if (!(r_e->uni_domain_name = (UNISTR2 *)talloc(ctx,sizeof(UNISTR2))))
+			return;
 
-		if (!(r_e->domain_sid = (DOM_SID2 *)
-		      malloc(sizeof(DOM_SID2)))) return;
+		if (!(r_e->domain_sid = (DOM_SID2 *)talloc(ctx,sizeof(DOM_SID2))))
+			return;
 
 		init_uni_hdr2(&r_e->hdr_domain_name[0], len_domain_name);
 		init_unistr2 (&r_e->uni_domain_name[0], domain_name, 
