@@ -56,7 +56,11 @@ uint32 _dfs_add(pipes_struct *p, DFS_Q_DFS_ADD* q_u, DFS_R_DFS_ADD *r_u)
 
   if (user.uid != 0) {
 	DEBUG(10,("_dfs_add: uid != 0. Access denied.\n"));
-	return ERROR_ACCESS_DENIED;
+
+	/* RPC calls return Windows errors. NT_STATUS_ACCESS_DENIED 
+	   doesn't work as a return code for RPC calls
+	*/
+	return ERRnoaccess;
   }
 
   unistr2_to_ascii(dfspath, &q_u->DfsEntryPath, sizeof(dfspath)-1);
@@ -70,10 +74,7 @@ uint32 _dfs_add(pipes_struct *p, DFS_Q_DFS_ADD* q_u, DFS_R_DFS_ADD *r_u)
   pstrcat(altpath, "\\");
   pstrcat(altpath, sharename);
 
-  if(!create_junction(dfspath, &jn))
-    return NERR_DfsNoSuchServer;
-
-  if(get_referred_path(&jn))
+  if(get_referred_path(dfspath, &jn, NULL, NULL))
     {
       exists = True;
       jn.referral_count += 1;
@@ -106,7 +107,7 @@ uint32 _dfs_add(pipes_struct *p, DFS_Q_DFS_ADD* q_u, DFS_R_DFS_ADD *r_u)
   if(!create_msdfs_link(&jn, exists))
     return NERR_DfsCantCreateJunctionPoint;
 
-  return NT_STATUS_NOPROBLEMO;
+  return ERRsuccess;
 }
 
 uint32 _dfs_remove(pipes_struct *p, DFS_Q_DFS_REMOVE *q_u, DFS_R_DFS_REMOVE *r_u)
@@ -122,7 +123,10 @@ uint32 _dfs_remove(pipes_struct *p, DFS_Q_DFS_REMOVE *q_u, DFS_R_DFS_REMOVE *r_u
 
   if (user.uid != 0) {
 	DEBUG(10,("_dfs_remove: uid != 0. Access denied.\n"));
-	return ERROR_ACCESS_DENIED;
+	/* NT_STATUS_ACCESS_DENIED will not work as a status code
+	   for RPC calls 
+	*/
+	return ERRnoaccess;
   }
 
   unistr2_to_ascii(dfspath, &q_u->DfsEntryPath, sizeof(dfspath)-1);
@@ -137,16 +141,14 @@ uint32 _dfs_remove(pipes_struct *p, DFS_Q_DFS_REMOVE *q_u, DFS_R_DFS_REMOVE *r_u
       pstrcpy(altpath, servername);
       pstrcat(altpath, "\\");
       pstrcat(altpath, sharename);
+      strlower(altpath);
     }
 
   DEBUG(5,("init_reply_dfs_remove: Request to remove %s -> %s\\%s.\n",
 	   dfspath, servername, sharename));
 
-  if(!create_junction(dfspath, &jn))
-    return NERR_DfsNoSuchServer;
-
-  if(!get_referred_path(&jn))
-    return NERR_DfsNoSuchVolume;
+  if(!get_referred_path(dfspath, &jn, NULL, NULL))
+	  return NERR_DfsNoSuchVolume;
 
   /* if no server-share pair given, remove the msdfs link completely */
   if(!q_u->ptr_ServerName && !q_u->ptr_ShareName)
@@ -158,14 +160,18 @@ uint32 _dfs_remove(pipes_struct *p, DFS_Q_DFS_REMOVE *q_u, DFS_R_DFS_REMOVE *r_u
     {
       int i=0;
       /* compare each referral in the list with the one to remove */
+      DEBUG(10,("altpath: .%s. refcnt: %d\n", altpath, jn.referral_count));
       for(i=0;i<jn.referral_count;i++)
 	{
 	  pstring refpath;
 	  pstrcpy(refpath,jn.referral_list[i].alternate_path);
 	  trim_string(refpath, "\\", "\\");
+	  DEBUG(10,("_dfs_remove:  refpath: .%s.\n", refpath));
 	  if(strequal(refpath, altpath))
 	    {
 	      *(jn.referral_list[i].alternate_path)='\0';
+	      DEBUG(10,("_dfs_remove: Removal request matches referral %s\n",
+			refpath));
 	      found = True;
 	    }
 	}
@@ -185,7 +191,7 @@ uint32 _dfs_remove(pipes_struct *p, DFS_Q_DFS_REMOVE *q_u, DFS_R_DFS_REMOVE *r_u
 	}
     }
 
-  return NT_STATUS_NOPROBLEMO;
+  return ERRsuccess;
 }
 
 static BOOL init_reply_dfs_info_1(struct junction_map* j, DFS_INFO_1* dfs1, int num_j)
@@ -227,8 +233,13 @@ static BOOL init_reply_dfs_info_3(TALLOC_CTX *ctx, struct junction_map* j, DFS_I
     {
       pstring str;
       dfs3[i].ptr_entrypath = 1;
-      slprintf(str, sizeof(pstring)-1, "\\\\%s\\%s\\%s", global_myname,
-	       j[i].service_name, j[i].volume_name);
+      if (j[i].volume_name[0] == '\0')
+	      slprintf(str, sizeof(pstring)-1, "\\\\%s\\%s",
+		       global_myname, j[i].service_name);
+      else
+	      slprintf(str, sizeof(pstring)-1, "\\\\%s\\%s\\%s", global_myname,
+		       j[i].service_name, j[i].volume_name);
+
       init_unistr2(&dfs3[i].entrypath, str, strlen(str)+1);
       dfs3[i].ptr_comment = 1;
       init_unistr2(&dfs3[i].comment, "", 1); 
@@ -309,7 +320,7 @@ static uint32 init_reply_dfs_ctr(TALLOC_CTX *ctx, uint32 level, DFS_INFO_CTR* ct
 	default:
 		return NT_STATUS_INVALID_LEVEL;
     }
-  return NT_STATUS_NOPROBLEMO;
+  return NT_STATUS_OK;
 }
       
 uint32 _dfs_enum(pipes_struct *p, DFS_Q_DFS_ENUM *q_u, DFS_R_DFS_ENUM *r_u)
@@ -353,7 +364,7 @@ uint32 _dfs_get_info(pipes_struct *p, DFS_Q_DFS_GET_INFO *q_u, DFS_R_DFS_GET_INF
   if(!create_junction(path, &jn))
      return NERR_DfsNoSuchServer;
   
-  if(!get_referred_path(&jn))
+  if(!get_referred_path(path, &jn, NULL, NULL))
     return NERR_DfsNoSuchVolume;
 
   r_u->level = level;
