@@ -251,8 +251,10 @@ uint32 local_lookup_alias_name(uint32 rid, char *alias_name, uint32 *type)
  ********************************************************************/
 uint32 local_lookup_user_name(uint32 rid, char *user_name, uint32 *type)
 {
-	struct sam_disp_info *disp_info;
+	SAM_ACCOUNT *sampwd=NULL;
 	int i = 0;
+	BOOL ret;
+	
 	(*type) = SID_NAME_USER;
 
 	DEBUG(5,("lookup_user_name: rid: %d", rid));
@@ -263,26 +265,28 @@ uint32 local_lookup_user_name(uint32 rid, char *user_name, uint32 *type)
 		i++;
 	}
 
-	if (domain_user_rids[i].rid != 0)
-	{
+	if (domain_user_rids[i].rid != 0) {
 		fstrcpy(user_name, domain_user_rids[i].name);
 		DEBUG(5,(" = %s\n", user_name));
 		return 0x0;
 	}
 
+	pdb_init_sam(&sampwd);
+
 	/* ok, it's a user.  find the user account */
 	become_root();
-	disp_info = getsamdisprid(rid);
+	ret = pdb_getsampwrid(sampwd, rid);
 	unbecome_root();
 
-	if (disp_info != NULL)
-	{
-		fstrcpy(user_name, disp_info->smb_name);
+	if (ret == True) {
+		fstrcpy(user_name, pdb_get_username(sampwd) );
 		DEBUG(5,(" = %s\n", user_name));
+		pdb_free_sam(sampwd);
 		return 0x0;
 	}
 
 	DEBUG(5,(" none mapped\n"));
+	pdb_free_sam(sampwd);
 	return NT_STATUS_NONE_MAPPED;
 }
 
@@ -329,19 +333,24 @@ uint32 local_lookup_alias_rid(char *alias_name, uint32 *rid)
  ********************************************************************/
 uint32 local_lookup_user_rid(char *user_name, uint32 *rid)
 {
-	struct sam_passwd *sam_pass;
+	SAM_ACCOUNT *sampass=NULL;
+	BOOL ret;
+
 	(*rid) = 0;
 
+	pdb_init_sam(&sampass);
+
 	/* find the user account */
 	become_root();
-	sam_pass = getsam21pwnam(user_name);
+	ret = pdb_getsampwnam(sampass, user_name);
 	unbecome_root();
 
-	if (sam_pass != NULL)
-	{
-		(*rid) = sam_pass->user_rid;
+	if (ret == True) {
+		(*rid) = pdb_get_user_rid(sampass);
+		pdb_free_sam(sampass);
 		return 0x0;
 	}
 
+	pdb_free_sam(sampass);
 	return NT_STATUS_NONE_MAPPED;
 }
