@@ -23,7 +23,7 @@
 #include "includes.h"
 
 extern int DEBUGLEVEL;
-extern fstring debugf;
+extern pstring debugf;
 
 DOM_SID domain_sid;
 
@@ -212,13 +212,13 @@ void fetch_domain_sid(struct cli_state *cli)
 	
 	if ((result = cli_lsa_open_policy(cli, mem_ctx, True, 
 					  SEC_RIGHTS_MAXIMUM_ALLOWED,
-					  &pol) != NT_STATUS_NOPROBLEMO)) {
+					  &pol) != NT_STATUS_OK)) {
 		goto error;
 	}
 
 	if ((result = cli_lsa_query_info_policy(cli, mem_ctx, &pol, info_class, 
 						domain_name, &domain_sid))
-	    != NT_STATUS_NOPROBLEMO) {
+	    != NT_STATUS_OK) {
 		goto error;
 	}
 
@@ -233,7 +233,7 @@ void fetch_domain_sid(struct cli_state *cli)
  error:
 	fprintf(stderr, "could not obtain sid for domain %s\n", cli->domain);
 
-	if (result != NT_STATUS_NOPROBLEMO) {
+	if (result != NT_STATUS_OK) {
 		fprintf(stderr, "error: %s\n", get_nt_error_msg(result));
 	}
 
@@ -279,7 +279,7 @@ static uint32 cmd_debuglevel(struct cli_state *cli, int argc, char **argv)
 {
 	if (argc > 2) {
 		printf("Usage: %s [debuglevel]\n", argv[0]);
-		return NT_STATUS_NOPROBLEMO;
+		return NT_STATUS_OK;
 	}
 
 	if (argc == 2) {
@@ -288,13 +288,13 @@ static uint32 cmd_debuglevel(struct cli_state *cli, int argc, char **argv)
 
 	printf("debuglevel is %d\n", DEBUGLEVEL);
 
-	return NT_STATUS_NOPROBLEMO;
+	return NT_STATUS_OK;
 }
 
 static uint32 cmd_quit(struct cli_state *cli, int argc, char **argv)
 {
 	exit(0);
-	return NT_STATUS_NOPROBLEMO; /* NOTREACHED */
+	return NT_STATUS_OK; /* NOTREACHED */
 }
 
 /* Build in rpcclient commands */
@@ -501,9 +501,9 @@ struct cli_state *setup_connection(struct cli_state *cli, char *system_name,
 
 
 /* Print usage information */
-static void usage(char *pname)
+static void usage(void)
 {
-	printf("Usage: %s server [options]\n", pname);
+	printf("Usage: rpcclient server [options]\n");
 
 	printf("\t-A authfile           file containing user credentials\n");
 	printf("\t-c \"command string\"   execute semicolon separated cmds\n");
@@ -543,17 +543,6 @@ static void usage(char *pname)
 
 	DEBUGLEVEL = 1;
 
-	/* Parse options */
-	if (argc < 2) {
-		usage(argv[0]);
-		return 0;
-	}
-
-	pstrcpy(server, argv[1]);
-
-	argv++;
-	argc--;
-
 	while ((opt = getopt(argc, argv, "A:s:Nd:U:W:c:l:")) != EOF) {
 		switch (opt) {
 		case 'A':
@@ -602,11 +591,25 @@ static void usage(char *pname)
 			
 		case 'h':
 		default:
-			usage(argv[0]);
+			usage();
 			exit(1);
 		}
 	}
 	
+	argv += optind;
+	argc -= optind;
+ 
+	/* Parse options */
+	if (argc == 0) {
+		usage();
+		return 0;
+	}
+ 
+	if (strncmp("//", argv[0], 2) == 0 || strncmp("\\\\", argv[0], 2) == 0)
+		argv[0] += 2;
+ 
+	pstrcpy(server, argv[0]);
+
 	/* the following functions are part of the Samba debugging
 	   facilities.  See lib/debug.c */
 	setup_logging (argv[0], interactive);
@@ -643,12 +646,12 @@ static void usage(char *pname)
 	else {
 		init_rpcclient_creds (&creds, username, domain, password);
 	}
-	memset(password,'X',strlen(password));
+	memset(password,'X',sizeof(password));
 
 	/* open a connection to the specified server */
 	ZERO_STRUCTP (&cli);
 	if (!setup_connection (&cli, server, &creds)) {
-		return 0;
+		return 1;
 	}
 	
 	/* There are no pointers in ntuser_creds struct so zero it out */
