@@ -37,7 +37,7 @@ SMB_OFF_T seek_file(files_struct *fsp,SMB_OFF_T pos)
   if (fsp->print_file && lp_postscript(fsp->conn->service))
     offset = 3;
 
-  seek_ret = sys_lseek(fsp->fd_ptr->fd,pos+offset,SEEK_SET);
+  seek_ret = fsp->conn->vfs_ops.lseek(fsp,fsp->fd,pos+offset,SEEK_SET);
 
   /*
    * We want to maintain the fiction that we can seek
@@ -69,7 +69,6 @@ SMB_OFF_T seek_file(files_struct *fsp,SMB_OFF_T pos)
  Read from write cache if we can.
 ****************************************************************************/
 
-static unsigned int cache_read_hits;
 
 BOOL read_from_write_cache(files_struct *fsp,char *data,SMB_OFF_T pos,size_t n)
 {
@@ -83,7 +82,7 @@ BOOL read_from_write_cache(files_struct *fsp,char *data,SMB_OFF_T pos,size_t n)
 
   memcpy(data, wcp->data + (pos - wcp->offset), n);
 
-  cache_read_hits++;
+  DO_PROFILE_INC(writecache_read_hits);
 
   return True;
 }
@@ -96,15 +95,10 @@ ssize_t read_file(files_struct *fsp,char *data,SMB_OFF_T pos,size_t n)
 {
   ssize_t ret=0,readret;
 
-#if USE_READ_PREDICTION
-  if (!fsp->can_write) {
-    ret = read_predict(fsp->fd_ptr->fd,pos,data,NULL,n);
-
-    data += ret;
-    n -= ret;
-    pos += ret;
+  /* you can't read from print files */
+  if (fsp->print_file) {
+	  return -1;
   }
-#endif
 
   /*
    * Serve from write cache if we can.
@@ -120,30 +114,20 @@ ssize_t read_file(files_struct *fsp,char *data,SMB_OFF_T pos,size_t n)
   }
   
   if (n > 0) {
-    readret = read(fsp->fd_ptr->fd,data,n);
+    readret = fsp->conn->vfs_ops.read(fsp,fsp->fd,data,n);
+    if (readret == -1)
+      return -1;
     if (readret > 0) ret += readret;
   }
 
   return(ret);
 }
 
-/* Write cache static counters. */
-
-static unsigned int abutted_writes;
-static unsigned int total_writes;
-static unsigned int non_oplock_writes;
-static unsigned int direct_writes;
-static unsigned int init_writes;
-static unsigned int flushed_writes;
-static unsigned int num_perfect_writes;
-static unsigned int flush_reasons[NUM_FLUSH_REASONS];
-
 /* how many write cache buffers have been allocated */
 static unsigned int allocated_write_caches;
-static unsigned int num_write_caches;
 
 /****************************************************************************
- *Really* write to a file
+ *Really* write to a file.
 ****************************************************************************/
 
 static ssize_t real_write_file(files_struct *fsp,char *data,SMB_OFF_T pos, size_t n)
@@ -151,7 +135,7 @@ static ssize_t real_write_file(files_struct *fsp,char *data,SMB_OFF_T pos, size_
   if ((pos != -1) && (seek_file(fsp,pos) == -1))
     return -1;
 
-  return write_data(fsp->fd_ptr->fd,data,n);
+  return vfs_write_data(fsp,data,n);
 }
 
 /****************************************************************************
@@ -164,6 +148,10 @@ ssize_t write_file(files_struct *fsp, char *data, SMB_OFF_T pos, size_t n)
   ssize_t total_written = 0;
   int write_path = -1; 
 
+  if (fsp->print_file) {
+	  return print_job_write(fsp->print_jobid, data, n);
+  }
+
   if (!fsp->can_write) {
     errno = EPERM;
     return(0);
@@ -173,7 +161,7 @@ ssize_t write_file(files_struct *fsp, char *data, SMB_OFF_T pos, size_t n)
     SMB_STRUCT_STAT st;
     fsp->modified = True;
 
-    if (sys_fstat(fsp->fd_ptr->fd,&st) == 0) {
+    if (fsp->conn->vfs_ops.fstat(fsp,fsp->fd,&st) == 0) {
       int dosmode = dos_mode(fsp->conn,fsp->fsp_name,&st);
       if (MAP_ARCHIVE(fsp->conn) && !IS_DOS_ARCHIVE(dosmode)) {	
         file_chmod(fsp->conn,fsp->fsp_name,dosmode | aARCH,&st);
@@ -184,17 +172,19 @@ ssize_t write_file(files_struct *fsp, char *data, SMB_OFF_T pos, size_t n)
        * the write cache.
        */
 
-      if ((fsp->oplock_type == EXCLUSIVE_OPLOCK) && !wcp) {
+      if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type) && !wcp) {
         setup_write_cache(fsp, st.st_size);
         wcp = fsp->wcp;
       } 
     }  
   }
 
-  total_writes++;
+#ifdef WITH_PROFILE
+  DO_PROFILE_INC(writecache_total_writes);
   if (!fsp->oplock_type) {
-    non_oplock_writes++;
+    DO_PROFILE_INC(writecache_non_oplock_writes);
   }
+#endif
 
   /*
    * If this file is level II oplocked then we need
@@ -204,105 +194,40 @@ ssize_t write_file(files_struct *fsp, char *data, SMB_OFF_T pos, size_t n)
    * the shared memory area whilst doing this.
    */
 
-  if (LEVEL_II_OPLOCK_TYPE(fsp->oplock_type)) {
-    SMB_DEV_T dev = fsp->fd_ptr->dev;
-    SMB_INO_T inode = fsp->fd_ptr->inode;
-    share_mode_entry *share_list = NULL;
-    pid_t pid = getpid();
-    int token = -1;
-    int num_share_modes = 0;
-    int i;
+  release_level_2_oplocks_on_change(fsp);
 
-    if (lock_share_entry(fsp->conn, dev, inode, &token) == False) {
-      DEBUG(0,("write_file: failed to lock share mode entry for file %s.\n", fsp->fsp_name ));
-    }
-
-    num_share_modes = get_share_modes(fsp->conn, token, dev, inode, &share_list);
-
-    for(i = 0; i < num_share_modes; i++) {
-      share_mode_entry *share_entry = &share_list[i];
-
-      /*
-       * As there could have been multiple writes waiting at the lock_share_entry
-       * gate we may not be the first to enter. Hence the state of the op_types
-       * in the share mode entries may be partly NO_OPLOCK and partly LEVEL_II
-       * oplock. It will do no harm to re-send break messages to those smbd's
-       * that are still waiting their turn to remove their LEVEL_II state, and
-       * also no harm to ignore existing NO_OPLOCK states. JRA.
-       */
-
-      if (share_entry->op_type == NO_OPLOCK)
-        continue;
-
-      /* Paranoia .... */
-      if (EXCLUSIVE_OPLOCK_TYPE(share_entry->op_type)) {
-        DEBUG(0,("write_file: PANIC. share mode entry %d is an exlusive oplock !\n", i ));
-        unlock_share_entry(fsp->conn, dev, inode, token);
-        abort();
-      }
-
-      /*
-       * Check if this is a file we have open (including the
-       * file we've been called to do write_file on. If so
-       * then break it directly without releasing the lock.
-       */
-
-      if (pid == share_entry->pid) {
-        files_struct *new_fsp = file_find_dit(dev, inode, &share_entry->time);
-
-        /* Paranoia check... */
-        if(new_fsp == NULL) {
-          DEBUG(0,("write_file: PANIC. share mode entry %d is not a local file !\n", i ));
-          unlock_share_entry(fsp->conn, dev, inode, token);
-          abort();
-        }
-        oplock_break_level2(new_fsp, True, token);
-
-      } else {
-
-        /*
-         * This is a remote file and so we send an asynchronous
-         * message.
-         */
-
-        request_oplock_break(share_entry, dev, inode);
-      }
-    }
- 
-    free((char *)share_list);
-    unlock_share_entry(fsp->conn, dev, inode, token);
-  }
-
-  /* Paranoia check... */
-  if (LEVEL_II_OPLOCK_TYPE(fsp->oplock_type)) {
-    DEBUG(0,("write_file: PANIC. File %s still has a level II oplock.\n", fsp->fsp_name));
-    abort();
-  }
-
-  if (total_writes % 500 == 0) {
-    DEBUG(3,("WRITECACHE: initwrites=%u abutted=%u flushes=%u total=%u \
+#ifdef WITH_PROFILE
+  if (profile_p && profile_p->writecache_total_writes % 500 == 0) {
+    DEBUG(3,("WRITECACHE: initwrites=%u abutted=%u total=%u \
 nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
-         init_writes, abutted_writes, flushed_writes, total_writes, non_oplock_writes,
-         allocated_write_caches,
-         num_write_caches, direct_writes, num_perfect_writes, cache_read_hits ));
+	profile_p->writecache_init_writes,
+	profile_p->writecache_abutted_writes,
+	profile_p->writecache_total_writes,
+	profile_p->writecache_non_oplock_writes,
+	profile_p->writecache_allocated_write_caches,
+	profile_p->writecache_num_write_caches,
+	profile_p->writecache_direct_writes,
+	profile_p->writecache_num_perfect_writes,
+	profile_p->writecache_read_hits ));
 
-    DEBUG(3,("WRITECACHE: SEEK=%d, READ=%d, WRITE=%d, READRAW=%d, OPLOCK=%d, CLOSE=%d, SYNC=%d\n",
-    flush_reasons[SEEK_FLUSH],
-    flush_reasons[READ_FLUSH],
-    flush_reasons[WRITE_FLUSH],
-    flush_reasons[READRAW_FLUSH],
-    flush_reasons[OPLOCK_RELEASE_FLUSH],
-    flush_reasons[CLOSE_FLUSH],
-    flush_reasons[SYNC_FLUSH] ));
+    DEBUG(3,("WRITECACHE: Flushes SEEK=%d, READ=%d, WRITE=%d, READRAW=%d, OPLOCK=%d, CLOSE=%d, SYNC=%d\n",
+	profile_p->writecache_flushed_writes[SEEK_FLUSH],
+	profile_p->writecache_flushed_writes[READ_FLUSH],
+	profile_p->writecache_flushed_writes[WRITE_FLUSH],
+	profile_p->writecache_flushed_writes[READRAW_FLUSH],
+	profile_p->writecache_flushed_writes[OPLOCK_RELEASE_FLUSH],
+	profile_p->writecache_flushed_writes[CLOSE_FLUSH],
+	profile_p->writecache_flushed_writes[SYNC_FLUSH] ));
   }
+#endif
 
   if(!wcp) {
-    direct_writes++;
+    DO_PROFILE_INC(writecache_direct_writes);
     return real_write_file(fsp, data, pos, n);
   }
 
   DEBUG(9,("write_file(fd=%d pos=%d size=%d) wofs=%d wsize=%d\n",
-       fsp->fd_ptr->fd, (int)pos, (int)n, (int)wcp->offset, (int)wcp->data_size));
+	   fsp->fd, (int)pos, (int)n, (int)wcp->offset, (int)wcp->data_size));
 
   /* 
    * If we have active cache and it isn't contiguous then we flush.
@@ -349,7 +274,7 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
       pos += data_used;
       n -= data_used;
 
-      abutted_writes++;
+      DO_PROFILE_INC(writecache_abutted_writes);
       total_written = data_used;
 
       write_path = 1;
@@ -385,7 +310,7 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
 
       cache_flush_needed = True;
 
-      abutted_writes++;
+      DO_PROFILE_INC(writecache_abutted_writes);
       total_written = data_used;
 
       write_path = 2;
@@ -428,14 +353,6 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
 
       wcp->file_size = wcp->offset + wcp->data_size;
 
-#if 0
-      if (set_filelen(fsp->fd_ptr->fd, wcp->file_size) == -1) {
-        DEBUG(0,("write_file: error %s in setting file to length %.0f\n",
-          strerror(errno), (double)wcp->file_size ));
-        return -1;
-      }
-#endif
-
       /*
        * If we used all the data then
        * return here.
@@ -455,7 +372,7 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
       pos += data_used;
       n -= data_used;
 
-      abutted_writes++;
+      DO_PROFILE_INC(writecache_abutted_writes);
       total_written = data_used;
 
       write_path = 3;
@@ -468,7 +385,7 @@ nonop=%u allocated=%u active=%u direct=%u perfect=%u readhits=%u\n",
        */
 
       DEBUG(9,("write_file: non cacheable write : fd = %d, pos = %.0f, len = %u, current cache pos = %.0f \
-len = %u\n",fsp->fd_ptr->fd, (double)pos, (unsigned int)n, (double)wcp->offset, (unsigned int)wcp->data_size ));
+len = %u\n",fsp->fd, (double)pos, (unsigned int)n, (double)wcp->offset, (unsigned int)wcp->data_size ));
 
       /*
        * Update the file size if needed.
@@ -487,7 +404,7 @@ len = %u\n",fsp->fd_ptr->fd, (double)pos, (unsigned int)n, (double)wcp->offset,
       if ( n <= wcp->alloc_size && n > wcp->data_size) {
         cache_flush_needed = True;
       } else {
-        direct_writes++;
+	DO_PROFILE_INC(writecache_direct_writes);
         return real_write_file(fsp, data, pos, n);
       }
 
@@ -499,11 +416,9 @@ len = %u\n",fsp->fd_ptr->fd, (double)pos, (unsigned int)n, (double)wcp->offset,
       wcp->file_size = wcp->data_size;
 
     if (cache_flush_needed) {
-      flushed_writes++;
-
       DEBUG(3,("WRITE_FLUSH:%d: due to noncontinuous write: fd = %d, size = %.0f, pos = %.0f, \
 n = %u, wcp->offset=%.0f, wcp->data_size=%u\n",
-             write_path, fsp->fd_ptr->fd, (double)wcp->file_size, (double)pos, (unsigned int)n,
+             write_path, fsp->fd, (double)wcp->file_size, (double)pos, (unsigned int)n,
              (double)wcp->offset, (unsigned int)wcp->data_size ));
 
       flush_write_cache(fsp, WRITE_FLUSH);
@@ -518,7 +433,7 @@ n = %u, wcp->offset=%.0f, wcp->data_size=%u\n",
   if (n > wcp->alloc_size ) {
     if(real_write_file(fsp, data, pos, n) == -1)
       return -1;
-    direct_writes++;
+    DO_PROFILE_INC(writecache_direct_writes);
     return total_written + n;
   }
 
@@ -527,16 +442,17 @@ n = %u, wcp->offset=%.0f, wcp->data_size=%u\n",
    */
 
   if (n) {
+#ifdef WITH_PROFILE
     if (wcp->data_size) {
-      abutted_writes++;
-      DEBUG(9,("abutted write (%u)\n", abutted_writes));
+      DO_PROFILE_INC(writecache_abutted_writes);
     } else {
-      init_writes++;
+      DO_PROFILE_INC(writecache_init_writes);
     }
+#endif
     memcpy(wcp->data+wcp->data_size, data, n);
     if (wcp->data_size == 0) {
       wcp->offset = pos;
-      num_write_caches++;
+      DO_PROFILE_INC(writecache_num_write_caches);
     }
     wcp->data_size += n;
     DEBUG(9,("cache return %u\n", (unsigned int)n));
@@ -561,6 +477,7 @@ void delete_write_cache(files_struct *fsp)
   if(!(wcp = fsp->wcp))
     return;
 
+  DO_PROFILE_DEC(writecache_allocated_write_caches);
   allocated_write_caches--;
 
   SMB_ASSERT(wcp->data_size == 0);
@@ -569,6 +486,9 @@ void delete_write_cache(files_struct *fsp)
   free(wcp);
 
   fsp->wcp = NULL;
+
+  DEBUG(10,("delete_write_cache: File %s deleted write cache\n", fsp->fsp_name ));
+
 }
 
 /****************************************************************************
@@ -580,7 +500,8 @@ static BOOL setup_write_cache(files_struct *fsp, SMB_OFF_T file_size)
   ssize_t alloc_size = lp_write_cache_size(SNUM(fsp->conn));
   write_cache *wcp;
 
-  if (allocated_write_caches >= MAX_WRITE_CACHES) return False;
+  if (allocated_write_caches >= MAX_WRITE_CACHES) 
+	return False;
 
   if(alloc_size == 0 || fsp->wcp)
     return False;
@@ -602,8 +523,12 @@ static BOOL setup_write_cache(files_struct *fsp, SMB_OFF_T file_size)
   }
 
   fsp->wcp = wcp;
+  DO_PROFILE_INC(writecache_allocated_write_caches);
   allocated_write_caches++;
 
+  DEBUG(10,("setup_write_cache: File %s allocated write cache size %u\n",
+		fsp->fsp_name, wcp->alloc_size ));
+
   return True;
 }
 
@@ -634,28 +559,27 @@ ssize_t flush_write_cache(files_struct *fsp, enum flush_reason_enum reason)
   data_size = wcp->data_size;
   wcp->data_size = 0;
 
-  num_write_caches--;
-
-  flush_reasons[(int)reason]++;
+  DO_PROFILE_DEC_INC(writecache_num_write_caches,writecache_flushed_writes[reason]);
 
   DEBUG(9,("flushing write cache: fd = %d, off=%.0f, size=%u\n",
-       fsp->fd_ptr->fd, (double)wcp->offset, (unsigned int)data_size));
+	   fsp->fd, (double)wcp->offset, (unsigned int)data_size));
 
+#ifdef WITH_PROFILE
   if(data_size == wcp->alloc_size)
-    num_perfect_writes++;
+    DO_PROFILE_INC(writecache_num_perfect_writes);
+#endif
 
   return real_write_file(fsp, wcp->data, wcp->offset, data_size);
 }
+
 /*******************************************************************
 sync a file
 ********************************************************************/
 
 void sync_file(connection_struct *conn, files_struct *fsp)
 {
-    if(lp_strict_sync(SNUM(conn)) && fsp->fd_ptr != NULL) {
+    if(lp_strict_sync(SNUM(conn)) && fsp->fd != -1) {
       flush_write_cache(fsp, SYNC_FLUSH);
-#ifdef HAVE_FSYNC
-      fsync(fsp->fd_ptr->fd);
-#endif
+      conn->vfs_ops.fsync(fsp,fsp->fd);
     }
 }
