@@ -23,6 +23,7 @@
 #include "includes.h"
 
 extern int DEBUGLEVEL;
+extern uint32 global_client_caps;
 
 static struct cnotify_fns *cnotify;
 
@@ -62,8 +63,11 @@ static void change_notify_reply_packet(char *inbuf, uint32 error_code)
 	 * can even determine how MS failed to test stuff and why.... :-). JRA.
 	 */
 	
-	SSVAL(outbuf,smb_flg2, SVAL(outbuf,smb_flg2) | FLAGS2_32_BIT_ERROR_CODES);
-	ERROR(0,error_code);
+	if (global_client_caps & CAP_STATUS32) {
+		ERROR(0,error_code);
+	} else {
+		ERROR(ERRDOS,STATUS_NOTIFY_ENUM_DIR);
+	}
 
 	/*
 	 * Seems NT needs a transact command with an error code
@@ -71,7 +75,8 @@ static void change_notify_reply_packet(char *inbuf, uint32 error_code)
 	 */
 	set_message(outbuf,18,0,False);
 
-	send_smb(smbd_server_fd(),outbuf);
+	if (!send_smb(smbd_server_fd(),outbuf))
+		exit_server("change_notify_reply_packet: send_smb failed.\n");
 }
 
 /****************************************************************************
