@@ -54,7 +54,7 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
   char *pp = params;
   char *pd = pdata;
   int params_sent_thistime, data_sent_thistime, total_sent_thistime;
-  int alignment_offset = 3;
+  int alignment_offset = 1; /* JRA. This used to be 3. Set to 1 to make netmon parse ok. */
   int data_alignment_offset = 0;
 
   /* Initially set the wcnt area to be 10 - this is true for all
@@ -410,7 +410,7 @@ static BOOL get_lanman2_dir_entry(connection_struct *conn,
       {
 	/* Needed to show the msdfs symlinks as directories */
 	if(!lp_host_msdfs() || !lp_msdfs_root(SNUM(conn)) 
-	   || !is_msdfs_link(conn, pathreal))
+	   || !is_msdfs_link(conn, pathreal, NULL, NULL))
 	  {
 	    DEBUG(5,("get_lanman2_dir_entry:Couldn't stat [%s] (%s)\n",
 		     pathreal,strerror(errno)));
@@ -1179,7 +1179,7 @@ static int call_trans2qfsinfo(connection_struct *conn,
       int fstype_len;
       SIVAL(pdata,0,FILE_CASE_PRESERVED_NAMES|FILE_CASE_SENSITIVE_SEARCH|
 			FILE_DEVICE_IS_MOUNTED|
-            (lp_nt_acl_support() ? FILE_PERSISTENT_ACLS : 0)); /* FS ATTRIBUTES */
+            (lp_nt_acl_support(SNUM(conn)) ? FILE_PERSISTENT_ACLS : 0)); /* FS ATTRIBUTES */
 #if 0 /* Old code. JRA. */
       SIVAL(pdata,0,0x4006); /* FS ATTRIBUTES == long filenames supported? */
       SIVAL(pdata,0,0x700FF);
@@ -1297,318 +1297,302 @@ static int call_trans2qfilepathinfo(connection_struct *conn,
 				    char **pparams,char **ppdata,
 				    int total_data)
 {
-  int max_data_bytes = SVAL(inbuf, smb_mdrcnt);
-  char *params = *pparams;
-  char *pdata = *ppdata;
-  uint16 tran_call = SVAL(inbuf, smb_setup0);
-  uint16 info_level;
-  int mode=0;
-  SMB_OFF_T size=0;
-  unsigned int data_size;
-  SMB_STRUCT_STAT sbuf;
-  pstring fname1;
-  char *fname;
-  char *p;
-  int l;
-  SMB_OFF_T pos = 0;
-  BOOL bad_path = False;
-  BOOL delete_pending = False;
-  time_t c_time;
+	int max_data_bytes = SVAL(inbuf, smb_mdrcnt);
+	char *params = *pparams;
+	char *pdata = *ppdata;
+	uint16 tran_call = SVAL(inbuf, smb_setup0);
+	uint16 info_level;
+	int mode=0;
+	SMB_OFF_T size=0;
+	unsigned int data_size;
+	SMB_STRUCT_STAT sbuf;
+	pstring fname1;
+	char *fname;
+	char *p;
+	int l;
+	SMB_OFF_T pos = 0;
+	BOOL bad_path = False;
+	BOOL delete_pending = False;
+	time_t c_time;
 
-  if (tran_call == TRANSACT2_QFILEINFO) {
-    files_struct *fsp = file_fsp(params,0);
-    info_level = SVAL(params,2);
+	if (tran_call == TRANSACT2_QFILEINFO) {
+		files_struct *fsp = file_fsp(params,0);
+		info_level = SVAL(params,2);
 
-    DEBUG(3,("call_trans2qfilepathinfo: TRANSACT2_QFILEINFO: level = %d\n", info_level));
+		DEBUG(3,("call_trans2qfilepathinfo: TRANSACT2_QFILEINFO: level = %d\n", info_level));
 
-    if(fsp && (fsp->is_directory || fsp->stat_open)) {
-      /*
-       * This is actually a QFILEINFO on a directory
-       * handle (returned from an NT SMB). NT5.0 seems
-       * to do this call. JRA.
-       */
-      fname = fsp->fsp_name;
-      unix_convert(fname,conn,0,&bad_path,&sbuf);
-      if (!check_name(fname,conn) || 
-          (!VALID_STAT(sbuf) && vfs_stat(conn,fname,&sbuf))) {
-        DEBUG(3,("fileinfo of %s failed (%s)\n",fname,strerror(errno)));
-        if((errno == ENOENT) && bad_path)
-        {
-          unix_ERR_class = ERRDOS;
-          unix_ERR_code = ERRbadpath;
-        }
-        return(UNIXERROR(ERRDOS,ERRbadpath));
-      }
+		if(fsp && (fsp->is_directory || fsp->stat_open)) {
+			/*
+			 * This is actually a QFILEINFO on a directory
+			 * handle (returned from an NT SMB). NT5.0 seems
+			 * to do this call. JRA.
+			 */
+			fname = fsp->fsp_name;
+			unix_convert(fname,conn,0,&bad_path,&sbuf);
+			if (!check_name(fname,conn) || 
+					(!VALID_STAT(sbuf) && vfs_stat(conn,fname,&sbuf))) {
+				DEBUG(3,("fileinfo of %s failed (%s)\n",fname,strerror(errno)));
 
-      delete_pending = fsp->directory_delete_on_close;
+				if((errno == ENOENT) && bad_path) {
+					unix_ERR_class = ERRDOS;
+					unix_ERR_code = ERRbadpath;
+				}
+				return(UNIXERROR(ERRDOS,ERRbadpath));
+			}
 
-    } else {
-      /*
-       * Original code - this is an open file.
-       */
-      CHECK_FSP(fsp,conn);
-      CHECK_ERROR(fsp);
+			delete_pending = fsp->directory_delete_on_close;
 
-      fname = fsp->fsp_name;
-      if (vfs_fstat(fsp,fsp->fd,&sbuf) != 0) {
-        DEBUG(3,("fstat of fnum %d failed (%s)\n",fsp->fnum, strerror(errno)));
-        return(UNIXERROR(ERRDOS,ERRbadfid));
-      }
-      if((pos = fsp->conn->vfs_ops.lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1)
-        return(UNIXERROR(ERRDOS,ERRnoaccess));
+		} else {
+			/*
+			 * Original code - this is an open file.
+			 */
+			CHECK_FSP(fsp,conn);
 
-      delete_pending = fsp->delete_on_close;
-    }
-  } else {
-    /* qpathinfo */
-    info_level = SVAL(params,0);
+			fname = fsp->fsp_name;
+			if (vfs_fstat(fsp,fsp->fd,&sbuf) != 0) {
+				DEBUG(3,("fstat of fnum %d failed (%s)\n",fsp->fnum, strerror(errno)));
+				return(UNIXERROR(ERRDOS,ERRbadfid));
+			}
 
-    DEBUG(3,("call_trans2qfilepathinfo: TRANSACT2_QPATHINFO: level = %d\n", info_level));
+			if((pos = fsp->conn->vfs_ops.lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1)
+				return(UNIXERROR(ERRDOS,ERRnoaccess));
 
-    fname = &fname1[0];
-    pstrcpy(fname,&params[6]);
+			delete_pending = fsp->delete_on_close;
+		}
+	} else {
+		/* qpathinfo */
+		info_level = SVAL(params,0);
 
-    RESOLVE_DFSPATH(fname, conn, inbuf, outbuf);
+		DEBUG(3,("call_trans2qfilepathinfo: TRANSACT2_QPATHINFO: level = %d\n", info_level));
 
-    unix_convert(fname,conn,0,&bad_path,&sbuf);
-    if (!check_name(fname,conn) || 
-        (!VALID_STAT(sbuf) && vfs_stat(conn,fname,&sbuf))) {
-      DEBUG(3,("fileinfo of %s failed (%s)\n",fname,strerror(errno)));
-      if((errno == ENOENT) && bad_path)
-      {
-        unix_ERR_class = ERRDOS;
-        unix_ERR_code = ERRbadpath;
-      }
-      return(UNIXERROR(ERRDOS,ERRbadpath));
-    }
-  }
+		fname = &fname1[0];
+		pstrcpy(fname,&params[6]);
 
+		RESOLVE_DFSPATH(fname, conn, inbuf, outbuf);
 
-  DEBUG(3,("call_trans2qfilepathinfo %s level=%d call=%d total_data=%d\n",
-	   fname,info_level,tran_call,total_data));
+		unix_convert(fname,conn,0,&bad_path,&sbuf);
+		if (!check_name(fname,conn) || (!VALID_STAT(sbuf) && vfs_stat(conn,fname,&sbuf))) {
+			DEBUG(3,("fileinfo of %s failed (%s)\n",fname,strerror(errno)));
+			if((errno == ENOENT) && bad_path) {
+				unix_ERR_class = ERRDOS;
+				unix_ERR_code = ERRbadpath;
+			}
+			return(UNIXERROR(ERRDOS,ERRbadpath));
+		}
+	}
 
-  p = strrchr(fname,'/'); 
-  if (!p) 
-    p = fname;
-  else
-    p++;
-  l = strlen(p);  
-  mode = dos_mode(conn,fname,&sbuf);
-  size = sbuf.st_size;
-  if (mode & aDIR) size = 0;
+	DEBUG(3,("call_trans2qfilepathinfo %s level=%d call=%d total_data=%d\n",
+			fname,info_level,tran_call,total_data));
 
-  /* from now on we only want the part after the / */
-  fname = p;
+	p = strrchr(fname,'/'); 
+	if (!p) 
+		p = fname;
+	else
+		p++;
+	l = strlen(p);  
+	mode = dos_mode(conn,fname,&sbuf);
+	size = sbuf.st_size;
+	if (mode & aDIR)
+		size = 0;
+
+	/* from now on we only want the part after the / */
+	fname = p;
   
-  params = Realloc(*pparams,2);
-  if ( params == NULL ) {
-    return(ERROR(ERRDOS,ERRnomem));
-  }
-  *pparams	= params;
-  memset((char *)params,'\0',2);
-  data_size = max_data_bytes + 1024;
-  pdata = Realloc(*ppdata, data_size); 
-  if ( pdata == NULL ) {
-    return(ERROR(ERRDOS,ERRnomem));
-  }
-  *ppdata	= pdata;
+	params = Realloc(*pparams,2);
+	if ( params == NULL )
+		return(ERROR(ERRDOS,ERRnomem));
+	*pparams	= params;
+	memset((char *)params,'\0',2);
+	data_size = max_data_bytes + 1024;
+	pdata = Realloc(*ppdata, data_size); 
+	if ( pdata == NULL )
+		return(ERROR(ERRDOS,ERRnomem));
+	*ppdata	= pdata;
 
-  if (total_data > 0 && IVAL(pdata,0) == total_data) {
-    /* uggh, EAs for OS2 */
-    DEBUG(4,("Rejecting EA request with total_data=%d\n",total_data));
-    return(ERROR(ERRDOS,ERROR_EAS_NOT_SUPPORTED));
-  }
+	if (total_data > 0 && IVAL(pdata,0) == total_data) {
+		/* uggh, EAs for OS2 */
+		DEBUG(4,("Rejecting EA request with total_data=%d\n",total_data));
+		return(ERROR(ERRDOS,ERReasnotsupported));
+	}
 
-  memset((char *)pdata,'\0',data_size);
+	memset((char *)pdata,'\0',data_size);
 
-  c_time = get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(conn)));
+	c_time = get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(conn)));
 
-  if (lp_dos_filetime_resolution(SNUM(conn))) {
-    c_time &= ~1;
-    sbuf.st_atime &= ~1;
-    sbuf.st_mtime &= ~1;
-    sbuf.st_mtime &= ~1;
-  }
+	if (lp_dos_filetime_resolution(SNUM(conn))) {
+		c_time &= ~1;
+		sbuf.st_atime &= ~1;
+		sbuf.st_mtime &= ~1;
+		sbuf.st_mtime &= ~1;
+	}
 
-  switch (info_level) 
-    {
-    case SMB_INFO_STANDARD:
-    case SMB_INFO_QUERY_EA_SIZE:
-      data_size = (info_level==1?22:26);
-      put_dos_date2(pdata,l1_fdateCreation,c_time);
-      put_dos_date2(pdata,l1_fdateLastAccess,sbuf.st_atime);
-      put_dos_date2(pdata,l1_fdateLastWrite,sbuf.st_mtime); /* write time */
-      SIVAL(pdata,l1_cbFile,(uint32)size);
-      SIVAL(pdata,l1_cbFileAlloc,SMB_ROUNDUP(size,1024));
-      SSVAL(pdata,l1_attrFile,mode);
-      SIVAL(pdata,l1_attrFile+2,4); /* this is what OS2 does */
-      break;
-
-    case SMB_INFO_QUERY_EAS_FROM_LIST:
-      data_size = 24;
-      put_dos_date2(pdata,0,c_time);
-      put_dos_date2(pdata,4,sbuf.st_atime);
-      put_dos_date2(pdata,8,sbuf.st_mtime);
-      SIVAL(pdata,12,(uint32)size);
-      SIVAL(pdata,16,SMB_ROUNDUP(size,1024));
-      SIVAL(pdata,20,mode);
-      break;
-
-    case SMB_INFO_QUERY_ALL_EAS:
-      data_size = 4;
-      SIVAL(pdata,0,data_size);
-      break;
-
-    case 6:
-      return(ERROR(ERRDOS,ERRbadfunc)); /* os/2 needs this */      
-
-    case SMB_QUERY_FILE_BASIC_INFO:
-	case 1004:
-
-      if (info_level == SMB_QUERY_FILE_BASIC_INFO)
-	      data_size = 36; /* w95 returns 40 bytes not 36 - why ?. */
-      else {
-          data_size = 40;
-          SIVAL(pdata,36,0);
-      }
-      put_long_date(pdata,c_time);
-      put_long_date(pdata+8,sbuf.st_atime);
-      put_long_date(pdata+16,sbuf.st_mtime); /* write time */
-      put_long_date(pdata+24,sbuf.st_mtime); /* change time */
-      SIVAL(pdata,32,mode);
-
-      DEBUG(5,("SMB_QFBI - "));
-      {
-        time_t create_time = c_time;
-        DEBUG(5,("create: %s ", ctime(&create_time)));
-      }
-      DEBUG(5,("access: %s ", ctime(&sbuf.st_atime)));
-      DEBUG(5,("write: %s ", ctime(&sbuf.st_mtime)));
-      DEBUG(5,("change: %s ", ctime(&sbuf.st_mtime)));
-      DEBUG(5,("mode: %x\n", mode));
-
-      break;
-
-    case SMB_QUERY_FILE_STANDARD_INFO:
-      data_size = 22;
-      SOFF_T(pdata,0,size);
-      SOFF_T(pdata,8,size);
-      SIVAL(pdata,16,sbuf.st_nlink);
-      CVAL(pdata,20) = 0;
-      CVAL(pdata,21) = (mode&aDIR)?1:0;
-      break;
-
-    case SMB_QUERY_FILE_EA_INFO:
-      data_size = 4;
-      break;
-
-    /* Get the 8.3 name - used if NT SMB was negotiated. */
-    case SMB_QUERY_FILE_ALT_NAME_INFO:
-      {
-        pstring short_name;
-        pstrcpy(short_name,p);
-        /* Mangle if not already 8.3 */
-        if(!is_8_3(short_name, True))
-        {
-          if(!name_map_mangle(short_name,True,True,SNUM(conn)))
-            *short_name = '\0';
-        }
-        strupper(short_name);
-        l = dos_PutUniCode(pdata + 4, short_name, sizeof(pstring), False);
-        data_size = 4 + l;
-        SIVAL(pdata,0,l);
-      }
-      break;
-
-    case SMB_QUERY_FILE_NAME_INFO:
-      /*
-       * The first part of this code is essential
-       * to get security descriptors to work on mapped
-       * drives. Don't ask how I discovered this unless
-       * you like hearing about me suffering.... :-). JRA.
-       */
-      if(strequal(".", fname) && (global_client_caps & CAP_UNICODE)) {
-        l = l*2;
-        SSVAL(outbuf,smb_flg2,SVAL(outbuf,smb_flg2)|FLAGS2_UNICODE_STRINGS);
-        dos_PutUniCode(pdata + 4, "\\",sizeof(pstring), False);
-      } else {
-        pstrcpy(pdata+4,fname);
-      }
-      data_size = 4 + l;
-      SIVAL(pdata,0,l);
-      break;
-
-    case SMB_QUERY_FILE_ALLOCATION_INFO:
-    case SMB_QUERY_FILE_END_OF_FILEINFO:
-      data_size = 8;
-      SOFF_T(pdata,0,size);
-      break;
-
-    case SMB_QUERY_FILE_ALL_INFO:
-      put_long_date(pdata,c_time);
-      put_long_date(pdata+8,sbuf.st_atime);
-      put_long_date(pdata+16,sbuf.st_mtime); /* write time */
-      put_long_date(pdata+24,sbuf.st_mtime); /* change time */
-      SIVAL(pdata,32,mode);
-      pdata += 40;
-      SOFF_T(pdata,0,size);
-      SOFF_T(pdata,8,size);
-      SIVAL(pdata,16,sbuf.st_nlink);
-      CVAL(pdata,20) = delete_pending;
-      CVAL(pdata,21) = (mode&aDIR)?1:0;
-      pdata += 24;
-      SINO_T(pdata,0,(SMB_INO_T)sbuf.st_ino); 
-      pdata += 8; /* index number */
-      pdata += 4; /* EA info */
-      if (mode & aRONLY)
-        SIVAL(pdata,0,0xA9);
-      else
-        SIVAL(pdata,0,0xd01BF);
-      pdata += 4;
-      SOFF_T(pdata,0,pos); /* current offset */
-      pdata += 8;
-      SIVAL(pdata,0,mode); /* is this the right sort of mode info? */
-      pdata += 4;
-      pdata += 4; /* alignment */
-      SIVAL(pdata,0,l);
-      pstrcpy(pdata+4,fname);
-      pdata += 4 + l;
-      data_size = PTR_DIFF(pdata,(*ppdata));
-      break;
-
-	/*
-	 * Windows 2000 completely undocumented new SMB info levels.
-	 * Thanks Microsoft.... sure you're working on making this
-	 * protocol a standard.... sure you are... :-).
-	 * Lying rat-bastards. JRA.
-	 */
-
-	case 1005:
-		SIVAL(pdata,0,mode);
-		SIVAL(pdata,4,0); /* ??? */
-		SOFF_T(pdata,8,size);
-		SIVAL(pdata,16,1); /* ??? */
-		SIVAL(pdata,20,0); /* ??? */
-		data_size = 24;
+	switch (info_level) {
+	case SMB_INFO_STANDARD:
+	case SMB_INFO_QUERY_EA_SIZE:
+		data_size = (info_level==1?22:26);
+		put_dos_date2(pdata,l1_fdateCreation,c_time);
+		put_dos_date2(pdata,l1_fdateLastAccess,sbuf.st_atime);
+		put_dos_date2(pdata,l1_fdateLastWrite,sbuf.st_mtime); /* write time */
+		SIVAL(pdata,l1_cbFile,(uint32)size);
+		SIVAL(pdata,l1_cbFileAlloc,SMB_ROUNDUP(size,1024));
+		SSVAL(pdata,l1_attrFile,mode);
+		SIVAL(pdata,l1_attrFile+2,4); /* this is what OS2 does */
 		break;
 
-	case 1006:
-		SIVAL(pdata,0,0x907); /* ??? */
-		SIVAL(pdata,4,0x690000); /* ??? */
+	case SMB_INFO_QUERY_EAS_FROM_LIST:
+		data_size = 24;
+		put_dos_date2(pdata,0,c_time);
+		put_dos_date2(pdata,4,sbuf.st_atime);
+		put_dos_date2(pdata,8,sbuf.st_mtime);
+		SIVAL(pdata,12,(uint32)size);
+		SIVAL(pdata,16,SMB_ROUNDUP(size,1024));
+		SIVAL(pdata,20,mode);
+		break;
+
+	case SMB_INFO_QUERY_ALL_EAS:
+		data_size = 4;
+		SIVAL(pdata,0,data_size);
+		break;
+
+	case 6:
+		return(ERROR(ERRDOS,ERRbadfunc)); /* os/2 needs this */      
+
+	case SMB_FILE_BASIC_INFORMATION:
+	case SMB_QUERY_FILE_BASIC_INFO:
+
+		if (info_level == SMB_QUERY_FILE_BASIC_INFO)
+			data_size = 36; /* w95 returns 40 bytes not 36 - why ?. */
+		else {
+			data_size = 40;
+			SIVAL(pdata,36,0);
+		}
+		put_long_date(pdata,c_time);
+		put_long_date(pdata+8,sbuf.st_atime);
+		put_long_date(pdata+16,sbuf.st_mtime); /* write time */
+		put_long_date(pdata+24,sbuf.st_mtime); /* change time */
+		SIVAL(pdata,32,mode);
+
+		DEBUG(5,("SMB_QFBI - "));
+
+		{
+			time_t create_time = c_time;
+			DEBUG(5,("create: %s ", ctime(&create_time)));
+		}
+
+		DEBUG(5,("access: %s ", ctime(&sbuf.st_atime)));
+		DEBUG(5,("write: %s ", ctime(&sbuf.st_mtime)));
+		DEBUG(5,("change: %s ", ctime(&sbuf.st_mtime)));
+		DEBUG(5,("mode: %x\n", mode));
+
+		break;
+
+	case SMB_FILE_STANDARD_INFORMATION:
+	case SMB_QUERY_FILE_STANDARD_INFO:
+
+		data_size = 22;
+		SOFF_T(pdata,0,size);
+		SOFF_T(pdata,8,size);
+		SIVAL(pdata,16,sbuf.st_nlink);
+		CVAL(pdata,20) = 0;
+		CVAL(pdata,21) = (mode&aDIR)?1:0;
+		break;
+
+	case SMB_FILE_EA_INFORMATION:
+	case SMB_QUERY_FILE_EA_INFO:
+		data_size = 4;
+		SIVAL(pdata,0,0);
+		break;
+
+		/* Get the 8.3 name - used if NT SMB was negotiated. */
+
+	case SMB_QUERY_FILE_ALT_NAME_INFO:
+	{
+		pstring short_name;
+		pstrcpy(short_name,p);
+		/* Mangle if not already 8.3 */
+		if(!is_8_3(short_name, True)) {
+			if(!name_map_mangle(short_name,True,True,SNUM(conn)))
+				*short_name = '\0';
+		}
+		strupper(short_name);
+		l = dos_PutUniCode(pdata + 4, short_name, sizeof(pstring), False);
+		data_size = 4 + l;
+		SIVAL(pdata,0,l);
+		break;
+	}
+
+	case SMB_QUERY_FILE_NAME_INFO:
+		/*
+		 * The first part of this code is essential
+		 * to get security descriptors to work on mapped
+		 * drives. Don't ask how I discovered this unless
+		 * you like hearing about me suffering.... :-). JRA.
+		 */
+
+		if(strequal(".", fname) && (global_client_caps & CAP_UNICODE)) {
+			l = l*2;
+			SSVAL(outbuf,smb_flg2,SVAL(outbuf,smb_flg2)|FLAGS2_UNICODE_STRINGS);
+			dos_PutUniCode(pdata + 4, "\\",sizeof(pstring), False);
+		} else {
+			pstrcpy(pdata+4,fname);
+		}
+		data_size = 4 + l;
+		SIVAL(pdata,0,l);
+		break;
+
+	case SMB_FILE_ALLOCATION_INFORMATION:
+	case SMB_FILE_END_OF_FILE_INFORMATION:
+	case SMB_QUERY_FILE_ALLOCATION_INFO:
+	case SMB_QUERY_FILE_END_OF_FILEINFO:
+		data_size = 8;
+		SOFF_T(pdata,0,size);
+		break;
+
+	case SMB_QUERY_FILE_ALL_INFO:
+		put_long_date(pdata,c_time);
+		put_long_date(pdata+8,sbuf.st_atime);
+		put_long_date(pdata+16,sbuf.st_mtime); /* write time */
+		put_long_date(pdata+24,sbuf.st_mtime); /* change time */
+		SIVAL(pdata,32,mode);
+		pdata += 40;
+		SOFF_T(pdata,0,size);
+		SOFF_T(pdata,8,size);
+		SIVAL(pdata,16,sbuf.st_nlink);
+		CVAL(pdata,20) = delete_pending;
+		CVAL(pdata,21) = (mode&aDIR)?1:0;
+		pdata += 24;
+		SINO_T(pdata,0,(SMB_INO_T)sbuf.st_ino); 
+		pdata += 8; /* index number */
+		pdata += 4; /* EA info */
+		if (mode & aRONLY)
+			SIVAL(pdata,0,0xA9);
+		else
+			SIVAL(pdata,0,0xd01BF);
+		pdata += 4;
+		SOFF_T(pdata,0,pos); /* current offset */
+		pdata += 8;
+		SIVAL(pdata,0,mode); /* is this the right sort of mode info? */
+		pdata += 4;
+		pdata += 4; /* alignment */
+		SIVAL(pdata,0,l);
+		pstrcpy(pdata+4,fname);
+		pdata += 4 + l;
+		data_size = PTR_DIFF(pdata,(*ppdata));
+		break;
+
+	case SMB_FILE_INTERNAL_INFORMATION:
+		/* This should be an index number - looks like dev/ino to me :-) */
+		SIVAL(pdata,0,sbuf.st_dev);
+		SIVAL(pdata,4,sbuf.st_ino);
 		data_size = 8;
 		break;
 
-	case 1007:
-		SIVAL(pdata,0,0); /* ??? */
-		data_size = 4;
-		break;
-
-	case 1008:
+	case SMB_FILE_ACCESS_INFORMATION:
 		SIVAL(pdata,0,0x12019F); /* ??? */
 		data_size = 4;
 		break;
 
-	case 1009:
+	case SMB_FILE_NAME_INFORMATION:
 		/* Pathname with leading '\'. */
 		{
 			pstring new_fname;
@@ -1622,19 +1606,23 @@ static int call_trans2qfilepathinfo(connection_struct *conn,
 			break;
 		}
 
-	case 1014:
-		SIVAL(pdata,0,0); /* ??? */
-		SIVAL(pdata,4,0); /* ??? */
-		data_size = 8;
+	case SMB_FILE_DISPOSITION_INFORMATION:
+		data_size = 1;
+		CVAL(pdata,0) = delete_pending;
 		break;
 
-	case 1016:
-		SIVAL(pdata,0,0); /* ??? */
+	case SMB_FILE_POSITION_INFORMATION:
+		data_size = 8;
+		SOFF_T(pdata,0,pos);
+		break;
+
+	case SMB_FILE_MODE_INFORMATION:
+		SIVAL(pdata,0,mode);
 		data_size = 4;
 		break;
 
-	case 1017:
-		SIVAL(pdata,0,0); /* ??? */
+	case SMB_FILE_ALIGNMENT_INFORMATION:
+		SIVAL(pdata,0,0); /* No alignment needed. */
 		data_size = 4;
 		break;
 
@@ -1663,7 +1651,7 @@ static int call_trans2qfilepathinfo(connection_struct *conn,
 		}
 #endif
 
-	case 1021:
+	case SMB_FILE_ALTERNATE_NAME_INFORMATION:
 		/* Last component of pathname. */
 		{
 			size_t byte_len = dos_PutUniCode(pdata+4,fname,max_data_bytes,False);
@@ -1672,8 +1660,10 @@ static int call_trans2qfilepathinfo(connection_struct *conn,
 			break;
 		}
 		
-	case 1022:
-		{
+	case SMB_FILE_STREAM_INFORMATION:
+		if (mode & aDIR) {
+			data_size = 0;
+		} else {
 			size_t byte_len = dos_PutUniCode(pdata+24,"::$DATA", 0xE, False);
 			SIVAL(pdata,0,0); /* ??? */
 			SIVAL(pdata,4,byte_len); /* Byte length of unicode string ::$DATA */
@@ -1681,59 +1671,57 @@ static int call_trans2qfilepathinfo(connection_struct *conn,
 			SIVAL(pdata,16,0x20); /* ??? */
 			SIVAL(pdata,20,0); /* ??? */
 			data_size = 24 + byte_len;
-			break;
 		}
+		break;
 
-	case 1028:
+	case SMB_FILE_COMPRESSION_INFORMATION:
 		SOFF_T(pdata,0,size);
 		SIVAL(pdata,8,0); /* ??? */
 		SIVAL(pdata,12,0); /* ??? */
 		data_size = 16;
 		break;
 
-	case 1034:
+	case SMB_FILE_NETWORK_OPEN_INFORMATION:
 		put_long_date(pdata,c_time);
 		put_long_date(pdata+8,sbuf.st_atime);
 		put_long_date(pdata+16,sbuf.st_mtime); /* write time */
 		put_long_date(pdata+24,sbuf.st_mtime); /* change time */
-		SIVAL(pdata,32,0x20); /* ??? */
-		SIVAL(pdata,36,0); /* ??? */
+		SOFF_T(pdata,32,(SMB_OFF_T)0x20); /* Allocation size. */
 		SOFF_T(pdata,40,size);
 		SIVAL(pdata,48,mode);
 		SIVAL(pdata,52,0); /* ??? */
 		data_size = 56;
 		break;
 
-	case 1035:
+	case SMB_FILE_ATTRIBUTE_TAG_INFORMATION:
 		SIVAL(pdata,0,mode);
 		SIVAL(pdata,4,0);
 		data_size = 8;
 		break;
 
+#if 0
 	/*
-	 * End new completely undocumented info levels... JRA.
+	 * NT4 server just returns "invalid query" to this - if we try to answer 
+	 * it then NTws gets a BSOD! (tridge).
 	 */
 
-#if 0
-      /* NT4 server just returns "invalid query" to this - if we try to answer 
-	 it then NTws gets a BSOD! (tridge) */
-    case SMB_QUERY_FILE_STREAM_INFO:
-      data_size = 24 + l;
-      SIVAL(pdata,0,pos);
-      SIVAL(pdata,4,size);
-      SIVAL(pdata,12,size);
-      SIVAL(pdata,20,l);	
-      pstrcpy(pdata+24,fname);
-      break;
+	case SMB_QUERY_FILE_STREAM_INFO:
+		data_size = 24 + l;
+		SIVAL(pdata,0,pos);
+		SIVAL(pdata,4,size);
+		SIVAL(pdata,12,size);
+		SIVAL(pdata,20,l);	
+		pstrcpy(pdata+24,fname);
+		break;
 #endif
 
-    default:
-      return(ERROR(ERRDOS,ERRunknownlevel));
-    }
+	default:
+		return(ERROR(ERRDOS,ERRunknownlevel));
+	}
 
-  send_trans2_replies( outbuf, bufsize, params, 2, *ppdata, data_size);
+	send_trans2_replies( outbuf, bufsize, params, 2, *ppdata, data_size);
 
-  return(-1);
+	return(-1);
 }
 
 /****************************************************************************
@@ -1779,11 +1767,12 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
         }
         return(UNIXERROR(ERRDOS,ERRbadpath));
       }
-    } else if (fsp->print_file) {
+    } else if (fsp && fsp->print_file) {
         /*
          * Doing a DELETE_ON_CLOSE should cancel a print job.
          */
-        if ((info_level == SMB_SET_FILE_DISPOSITION_INFO) && CVAL(pdata,0)) {
+        if (((info_level == SMB_SET_FILE_DISPOSITION_INFO)||(info_level == SMB_FILE_DISPOSITION_INFORMATION)) &&
+						CVAL(pdata,0)) {
           fsp->share_mode = FILE_DELETE_ON_CLOSE;
 
           DEBUG(3,("call_trans2setfilepathinfo: Cancelling print job (%s)\n",
@@ -1798,7 +1787,6 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
        * Original code - this is an open file.
        */
       CHECK_FSP(fsp,conn);
-      CHECK_ERROR(fsp);
 
       fname = fsp->fsp_name;
       fd = fsp->fd;
@@ -1858,7 +1846,7 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
   if (total_data > 4 && IVAL(pdata,0) == total_data) {
     /* uggh, EAs for OS2 */
     DEBUG(4,("Rejecting EA request with total_data=%d\n",total_data));
-    return(ERROR(ERRDOS,ERROR_EAS_NOT_SUPPORTED));
+    return(ERROR(ERRDOS,ERReasnotsupported));
   }
 
   switch (info_level)
@@ -1923,28 +1911,62 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
       break;
     }
 
-    /*
-     * NT seems to use this call with a size of zero
-     * to mean truncate the file. JRA.
-     */
-
-	case 1019:
-	case 1020:
+	case SMB_FILE_ALLOCATION_INFORMATION:
     case SMB_SET_FILE_ALLOCATION_INFO:
     {
-      SMB_OFF_T newsize = IVAL(pdata,0);
+      int ret = -1;
+      size = IVAL(pdata,0);
 #ifdef LARGE_SMB_OFF_T
-      newsize |= (((SMB_OFF_T)IVAL(pdata,4)) << 32);
+      size |= (((SMB_OFF_T)IVAL(pdata,4)) << 32);
 #else /* LARGE_SMB_OFF_T */
       if (IVAL(pdata,4) != 0)	/* more than 32 bits? */
          return(ERROR(ERRDOS,ERRunknownlevel));
 #endif /* LARGE_SMB_OFF_T */
-      DEBUG(10,("call_trans2setfilepathinfo: Set file allocation info for file %s to %.0f\n", fname, (double)newsize ));
-      if(newsize == 0)
-        size = 0;
+      DEBUG(10,("call_trans2setfilepathinfo: Set file allocation info for file %s to %.0f\n",
+                           fname, (double)size ));
+
+      if(size != sbuf.st_size) {
+ 
+        DEBUG(10,("call_trans2setfilepathinfo: file %s : setting new size to %.0f\n",
+            fname, (double)size ));
+ 
+        if (fd == -1) {
+          files_struct *new_fsp = NULL;
+          int access_mode = 0;
+          int action = 0;
+ 
+          if(global_oplock_break) {
+            /* Queue this file modify as we are the process of an oplock break.  */
+ 
+            DEBUG(2,("call_trans2setfilepathinfo: queueing message due to being "));
+            DEBUGADD(2,( "in oplock break state.\n"));
+ 
+            push_oplock_pending_smb_message(inbuf, length);
+            return -1;
+          }
+ 
+          new_fsp = open_file_shared(conn, fname, &sbuf,
+                             SET_OPEN_MODE(DOS_OPEN_RDWR),
+                             (FILE_FAIL_IF_NOT_EXIST|FILE_EXISTS_OPEN),
+                             0, 0, &access_mode, &action);
+ 
+          if (new_fsp == NULL)
+            return(UNIXERROR(ERRDOS,ERRbadpath));
+          ret = vfs_allocate_file_space(new_fsp, size);
+          close_file(new_fsp,True);
+        } else {
+          ret = vfs_allocate_file_space(fsp, size);
+        }
+        if (ret == -1)
+          return allocate_space_error(inbuf, outbuf, errno);
+
+        sbuf.st_size = size;
+      }
+
       break;
     }
 
+	case SMB_FILE_END_OF_FILE_INFORMATION:
     case SMB_SET_FILE_END_OF_FILE_INFO:
     {
       size = IVAL(pdata,0);
@@ -1958,6 +1980,7 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
       break;
     }
 
+    case SMB_FILE_DISPOSITION_INFORMATION:
     case SMB_SET_FILE_DISPOSITION_INFO: /* Set delete on close for open file. */
     {
 		BOOL delete_on_close = (CVAL(pdata,0) ? True : False);
@@ -2144,7 +2167,7 @@ static int call_trans2setfilepathinfo(connection_struct *conn,
       vfs_set_filelen(new_fsp, size);
       close_file(new_fsp,True);
     } else {
-        vfs_set_filelen(fsp, size);
+      vfs_set_filelen(fsp, size);
     }
   }
 
@@ -2351,8 +2374,9 @@ static int call_trans2ioctl(connection_struct *conn, char* inbuf,
 }
 
 /****************************************************************************
-  reply to a SMBfindclose (stop trans2 directory search)
+ Reply to a SMBfindclose (stop trans2 directory search).
 ****************************************************************************/
+
 int reply_findclose(connection_struct *conn,
 		    char *inbuf,char *outbuf,int length,int bufsize)
 {
@@ -2373,8 +2397,9 @@ int reply_findclose(connection_struct *conn,
 }
 
 /****************************************************************************
-  reply to a SMBfindnclose (stop FINDNOTIFYFIRST directory search)
+ Reply to a SMBfindnclose (stop FINDNOTIFYFIRST directory search).
 ****************************************************************************/
+
 int reply_findnclose(connection_struct *conn, 
 		     char *inbuf,char *outbuf,int length,int bufsize)
 {
@@ -2398,10 +2423,10 @@ int reply_findnclose(connection_struct *conn,
 	return(outsize);
 }
 
-
 /****************************************************************************
-  reply to a SMBtranss2 - just ignore it!
+ Reply to a SMBtranss2 - just ignore it!
 ****************************************************************************/
+
 int reply_transs2(connection_struct *conn,
 		  char *inbuf,char *outbuf,int length,int bufsize)
 {
