@@ -73,7 +73,7 @@ void smbw_init(void)
 		exit(1);
 	}
 
-	charset_initialise();
+	charset_initialize();
 
 	in_client = True;
 
@@ -85,6 +85,8 @@ void smbw_init(void)
 
 	lp_load(servicesf,True,False,False);
 
+	charset_initialize();
+
 	get_myname(global_myname);
 
 	if ((p=smbw_getshared("DEBUG"))) {
