@@ -78,7 +78,7 @@ static struct smbw_dir *cur_dir;
 /***************************************************** 
 add a entry to a directory listing
 *******************************************************/
-static void smbw_dir_add(struct file_info *finfo, const char *mask)
+static void smbw_dir_add(struct file_info *finfo, const char *mask, void *state)
 {
 	DEBUG(5,("%s\n", finfo->name));
 
@@ -101,7 +101,7 @@ static void smbw_dir_add(struct file_info *finfo, const char *mask)
 add a entry to a directory listing
 *******************************************************/
 static void smbw_share_add(const char *share, uint32 type, 
-                           const char *comment)
+                           const char *comment, void *state)
 {
 	struct file_info finfo;
 
@@ -112,7 +112,7 @@ static void smbw_share_add(const char *share, uint32 type,
 	pstrcpy(finfo.name, share);
 	finfo.mode = aRONLY | aDIR;	
 
-	smbw_dir_add(&finfo, NULL);
+	smbw_dir_add(&finfo, NULL, NULL);
 }
 
 
@@ -120,7 +120,7 @@ static void smbw_share_add(const char *share, uint32 type,
 add a server to a directory listing
 *******************************************************/
 static void smbw_server_add(const char *name, uint32 type, 
-			    const char *comment)
+			    const char *comment, void *state)
 {
 	struct file_info finfo;
 
@@ -129,7 +129,7 @@ static void smbw_server_add(const char *name, uint32 type,
 	pstrcpy(finfo.name, name);
 	finfo.mode = aRONLY | aDIR;	
 
-	smbw_dir_add(&finfo, NULL);
+	smbw_dir_add(&finfo, NULL, NULL);
 }
 
 
@@ -151,7 +151,7 @@ static void smbw_printjob_add(struct print_job_info *job)
 	finfo.mode = aRONLY;
 	finfo.size = job->size;
 
-	smbw_dir_add(&finfo, NULL);
+	smbw_dir_add(&finfo, NULL, NULL);
 }
 
 
@@ -202,30 +202,30 @@ int smbw_dir_open(const char *fname)
 
 	if ((p=strstr(srv->server_name,"#01"))) {
 		*p = 0;
-		smbw_server_add(".",0,"");
-		smbw_server_add("..",0,"");
+		smbw_server_add(".",0,"",NULL);
+		smbw_server_add("..",0,"",NULL);
 		cli_NetServerEnum(&srv->cli, srv->server_name, SV_TYPE_DOMAIN_ENUM,
-				  smbw_server_add);
+				  smbw_server_add,NULL);
 		*p = '#';
 	} else if ((p=strstr(srv->server_name,"#1D"))) {
 		DEBUG(4,("doing NetServerEnum\n"));
 		*p = 0;
-		smbw_server_add(".",0,"");
-		smbw_server_add("..",0,"");
+		smbw_server_add(".",0,"",NULL);
+		smbw_server_add("..",0,"",NULL);
 		cli_NetServerEnum(&srv->cli, srv->server_name, SV_TYPE_ALL,
-				  smbw_server_add);
+				  smbw_server_add,NULL);
 		*p = '#';
 	} else if (strcmp(srv->cli.dev,"IPC") == 0) {
 		DEBUG(4,("doing NetShareEnum\n"));
-		smbw_share_add(".",0,"");
-		smbw_share_add("..",0,"");
-		if (cli_RNetShareEnum(&srv->cli, smbw_share_add) < 0) {
+		smbw_share_add(".",0,"",NULL);
+		smbw_share_add("..",0,"",NULL);
+		if (cli_RNetShareEnum(&srv->cli, smbw_share_add,NULL) < 0) {
 			errno = smbw_errno(&srv->cli);
 			goto failed;
 		}
 	} else if (strncmp(srv->cli.dev,"LPT",3) == 0) {
-		smbw_share_add(".",0,"");
-		smbw_share_add("..",0,"");
+		smbw_share_add(".",0,"",NULL);
+		smbw_share_add("..",0,"",NULL);
 		if (cli_print_queue(&srv->cli, smbw_printjob_add) < 0) {
 			errno = smbw_errno(&srv->cli);
 			goto failed;
@@ -233,11 +233,11 @@ int smbw_dir_open(const char *fname)
 	} else {
 #if 0
 		if (strcmp(path,"\\") == 0) {
-			smbw_share_add(".",0,"");
-			smbw_share_add("..",0,"");
+			smbw_share_add(".",0,"",NULL);
+			smbw_share_add("..",0,"",NULL);
 		}
 #endif
-		if (cli_list(&srv->cli, mask, aHIDDEN|aSYSTEM|aDIR, smbw_dir_add) < 0) {
+		if (cli_list(&srv->cli, mask, aHIDDEN|aSYSTEM|aDIR, smbw_dir_add,NULL) < 0) {
 			errno = smbw_errno(&srv->cli);
 			goto failed;
 		}
