@@ -44,7 +44,7 @@ static void usage(BOOL doexit)
 {
 	int i;
 	if (doexit) {
-		printf("Usage: smbcontrol -i\n");
+		printf("Usage: smbcontrol -i -s configfile\n");
 		printf("       smbcontrol <destination> <message-type> <parameters>\n\n");
 	} else {
 		printf("<destination> <message-type> <parameters>\n\n");
@@ -125,10 +125,11 @@ send a message to a named destination
 ****************************************************************************/
 static BOOL send_message(char *dest, int msg_type, void *buf, int len, BOOL duplicates)
 {
-	pid_t pid;
+	BOOL retval = False;
+	pid_t pid = 0;
 	TDB_CONTEXT *the_tdb;
 
-	the_tdb = tdb_open_log(lock_path("connections.tdb"), 0, 0, O_RDONLY, 0);
+	the_tdb = tdb_open_log(lock_path("connections.tdb"), 0, TDB_DEFAULT, O_RDWR, 0);
 	if (!the_tdb) {
 		fprintf(stderr,"Failed to open connections database in send_message.\n");
 		return False;
@@ -136,12 +137,11 @@ static BOOL send_message(char *dest, int msg_type, void *buf, int len, BOOL dupl
 
 	/* "smbd" is the only broadcast operation */
 	if (strequal(dest,"smbd")) {
-		return message_send_all(the_tdb,msg_type, buf, len, duplicates);
+		retval = message_send_all(the_tdb,msg_type, buf, len, duplicates);
 	} else if (strequal(dest,"nmbd")) {
 		pid = pidfile_pid(dest);
 		if (pid == 0) {
 			fprintf(stderr,"Can't find pid for nmbd\n");
-			return False;
 		}
 	} else if (strequal(dest,"self")) {
 		pid = getpid();
@@ -149,11 +149,14 @@ static BOOL send_message(char *dest, int msg_type, void *buf, int len, BOOL dupl
 		pid = atoi(dest);
 		if (pid == 0) {
 			fprintf(stderr,"Not a valid pid\n");
-			return False;
 		}		
 	} 
 
-	return message_send_pid(pid, msg_type, buf, len, duplicates);
+	tdb_close(the_tdb);
+	if (pid)
+		return message_send_pid(pid, msg_type, buf, len, duplicates);
+	else
+		return retval;
 }
 
 /****************************************************************************
@@ -337,23 +340,27 @@ static BOOL do_command(char *dest, char *msg_name, char **params)
 	setup_logging(argv[0],True);
 	
 	charset_initialise();
-	lp_load(servicesf,False,False,False);
-
-	if (!message_init()) exit(1);
 
 	if (argc < 2) usage(True);
 
-	while ((opt = getopt(argc, argv,"i")) != EOF) {
+	while ((opt = getopt(argc, argv,"is:")) != EOF) {
 		switch (opt) {
 		case 'i':
 			interactive = True;
 			break;
+		case 's':
+			pstrcpy(servicesf, optarg);
+			break;
 		default:
 			printf("Unknown option %c (%d)\n", (char)opt, opt);
 			usage(True);
 		}
 	}
 
+	lp_load(servicesf,False,False,False);
+
+	if (!message_init()) exit(1);
+
 	argc -= optind;
 	argv = &argv[optind];
 
